import { Base64encrypt, Base64decrypt } from '@/utils/crypto'

const parserRules = [
  { attr: 'id' },
  { attr: 'moduleKey' },
  { attr: 'moduleSql', default: '', isCrypto: true, unJson: true },
  { attr: 'moduleName', default: '' },
  { attr: 'moduleCode', default: '' },
  { attr: 'moduleStatus' },
  { attr: 'remarks', default: '' },
  { attr: 'cacheSwitch' },
  { attr: 'cacheSecond' },
  { attr: 'triptoSource', default: '' },
  { attr: 'pagination', default: '' },
  { attr: 'pageSize' },
  { attr: 'sortProp', default: '' },
  { attr: 'sortType', default: '' },
  { attr: 'formInput', default: [], isCrypto: true },
  { attr: 'formBtn', default: [], isCrypto: true },
  { attr: 'tableColumn', default: [], isCrypto: true },
  { attr: 'tableBtn', default: [], isCrypto: true },
  { attr: 'customProperties', default: {}, isCrypto: true,
    child: [
      { attr: 'custom', default: {}},
      { attr: 'axis', default: [] },
      { attr: 'series', default: [] },
      { attr: 'module' },
      { attr: 'chartOption', default: {}},
      { attr: 'tableOption', default: {}}
    ]
  }
]

const matchRules = [
  { tab: 'module', attr: [
    { name: 'id' },
    { name: 'moduleKey' },
    { name: 'moduleSql' },
    { name: 'moduleName' },
    { name: 'moduleCode' },
    { name: 'moduleStatus' },
    { name: 'remarks' },
    { name: 'cacheSwitch' },
    { name: 'cacheSecond' },
    { name: 'triptoSource' },
    { name: 'pagination' },
    { name: 'pageSize' },
    { name: 'sortProp' },
    { name: 'sortType' },
    { name: 'moduleType', path: ['customProperties', 'module', 'moduleType'] }
  ] },
  { tab: 'custom', attr: { path: ['customProperties', 'custom'] }},
  { tab: 'formInput', attr: { path: ['formInput'] }},
  { tab: 'formButton', attr: { path: ['formBtn'] }},
  { tab: 'tableColumn', attr: { path: ['tableColumn'] }},
  { tab: 'tableButton', attr: { path: ['tableBtn'] }},
  { tab: 'axis', attr: { path: ['customProperties', 'axis'] }},
  { tab: 'series', attr: { path: ['customProperties', 'series'] }},
  { tab: 'chartOption', attr: { path: ['customProperties', 'chartOption'] }},
  { tab: 'tableOption', attr: { path: ['customProperties', 'tableOption'] }}
]

function decrypt(str) {
  return str !== undefined ? Base64decrypt(decodeURIComponent(str)) : str
}
function encrypt(str) {
  return str !== undefined ? encodeURIComponent(Base64encrypt(str)) : str
}
function dataParser(data, rules) {
  const temp = {}
  rules.forEach(r => {
    let value
    if (r.isCrypto) {
      const text = decrypt(data[r.attr])
      if (r.unJson || !text) {
        value = text
      } else {
        try {
          value = JSON.parse(text)
        } catch (error) {
          console.error(error)
        }
      }
    } else {
      value = data[r.attr]
    }
    if (value === undefined) {
      value = r.default
    } else if (r.type) {
      if (r.type === 'number') {
        value = Number(value)
      }
    }
    if (r.child) {
      value = dataParser(value, r.child)
    }
    temp[r.attr] = value
  })
  return temp
}
function dataMatch(data) {
  const temp = {}
  matchRules.forEach(r => {
    let value
    if (r.attr) {
      if (Array.isArray(r.attr)) {
        value = {}
        r.attr.forEach(attr => {
          let v
          if (attr.path) {
            v = attr.path.reduce((total, curr) => {
              if (!total) {
                total = {}
              }
              return total[curr]
            }, data)
          } else if (attr.name) {
            v = data[attr.name]
          }
          value[attr.name] = v
        })
      } else if (typeof r.attr === 'object') {
        value = r.attr.path.reduce((total, curr) => {
          if (!total) {
            total = {}
          }
          return total[curr]
        }, data)
      }
    }
    temp[r.tab] = value
  })
  return temp
}

export function parser(apiData) {
  return dataMatch(dataParser(Object.assign({}, apiData), parserRules))
}

export function packager(data) {
  const temp = {}
  matchRules.forEach(r => {
    if (Array.isArray(r.attr)) {
      r.attr.forEach(attr => {
        if (attr.path) {
          attr.path.reduce((total, curr, index) => {
            if (index === attr.path.length - 1) {
              total[curr] = data[r.tab][attr.name]
            } else if (!total[curr]) {
              total[curr] = {}
            }
            return total[curr]
          }, temp)
        } else if (attr.name) {
          temp[attr.name] = data[r.tab][attr.name]
        }
      })
    } else if (typeof r.attr === 'object') {
      if (r.attr.path) {
        r.attr.path.reduce((total, curr, index) => {
          if (index === r.attr.path.length - 1) {
            total[curr] = data[r.tab]
          } else if (!total[curr]) {
            total[curr] = {}
          }
          return total[curr]
        }, temp)
      }
    }
  })
  parserRules.forEach(r => {
    if (r.isCrypto) {
      temp[r.attr] = encrypt(temp[r.attr])
    }
  })
  return temp
}
